@@ -7,7 +7,7 @@
 #include <string>
 #include <tuple>
 
-namespace gl_wrap {
+namespace gl {
 
 class gl_exception : public std::exception
 {
@@ -132,6 +132,7 @@ enum class Primitive
     Triangles = GL_TRIANGLES,
     Lines = GL_LINES,
     LineLoop = GL_LINE_LOOP,
+    LineStrip = GL_LINE_STRIP,
     Points = GL_POINTS,
 };
 
@@ -140,6 +141,7 @@ enum class Type
     UnsignedInt = GL_UNSIGNED_INT,
     UnsignedShort = GL_UNSIGNED_SHORT,
     Float = GL_FLOAT,
+    Double = GL_DOUBLE,
 };
 
 template <typename T> inline T get(GLenum what)
