@@ -7,7 +7,7 @@
 #include <cassert>
 #include <vector>
 
-namespace gl_wrap {
+namespace gl {
 
 struct Program
 {
@@ -27,7 +27,7 @@ struct Program
         return *this;
     }
 
-    operator bool() const {
+    explicit operator bool() const {
         return program != 0;
     }
 
@@ -42,7 +42,10 @@ struct Program
 
     ~Program()
     {
-        if (program != 0) { glDeleteProgram(program); }
+        if (program != 0) { 
+            // TODO: Temp remove because cached programs outlive context
+            //glDeleteProgram(program);
+        }
     }
 
     Program(VertexShader const& vs, FragmentShader const& fs)
