@@ -9,7 +9,7 @@
 #include <memory>
 #include <vector>
 
-namespace gl_wrap {
+namespace gl {
 
 struct Texture
 {
@@ -32,6 +32,15 @@ struct Texture
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
     }
 
+    void set_filter(bool min, bool max)
+    {
+        glBindTexture(GL_TEXTURE_2D, tex_id);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
+                        min ? GL_LINEAR : GL_NEAREST);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
+                        max ? GL_LINEAR : GL_NEAREST);
+    }
+
     template <typename T, size_t N>
     Texture(GLint w, GLint h, std::array<T, N> const& data,
             GLint target_format = GL_RGBA, GLint source_format = -1,
@@ -148,9 +157,7 @@ struct Texture
 
     void set_target()
     {
-        if(!alloc_framebuffer()) {
-            glBindFramebuffer(GL_FRAMEBUFFER, fb_id);
-        }
+        if (!alloc_framebuffer()) { glBindFramebuffer(GL_FRAMEBUFFER, fb_id); }
         setViewport({width, height});
     }
 
@@ -189,9 +196,8 @@ struct Texture
         return data;
     }
 
-    template <typename T>
-    void update(T const* ptr, GLint source_format = -1,
-                GLenum type = GL_UNSIGNED_BYTE) const
+    template <typename T> void update(T const* ptr, GLint source_format = -1,
+                                      GLenum type = GL_UNSIGNED_BYTE) const
     {
         if (source_format < 0) {
             constexpr static std::array translate{0, GL_ALPHA, 0, GL_RGB,
@@ -203,9 +209,9 @@ struct Texture
                         type, ptr);
     }
 
-    template <typename T>
-    void update(int x, int y, int w, int h, T const* ptr,
-                GLint source_format = -1, GLenum type = GL_UNSIGNED_BYTE) const
+    template <typename T> void update(int x, int y, int w, int h, T const* ptr,
+                                      GLint source_format = -1,
+                                      GLenum type = GL_UNSIGNED_BYTE) const
     {
         if (source_format < 0) {
             constexpr static std::array translate{0, GL_ALPHA, 0, GL_RGB,
@@ -244,6 +250,8 @@ public:
     {
     }
 
+    void set_texture_filter(bool min, bool max) { tex->set_filter(min, max); }
+
     const auto& uvs() const { return _uvs; }
 
     void bind(int unit = 0) const { tex->bind(unit); }
@@ -255,12 +263,12 @@ public:
 
     operator bool() const { return tex != nullptr; }
 
-    void copy_from(gl_wrap::TexRef const& src) const
+    void copy_from(gl::TexRef const& src) const
     {
-        using gl_wrap::ProgramCache;
+        using gl::ProgramCache;
         GLint fb = 0;
         glGetIntegerv(GL_FRAMEBUFFER_BINDING, &fb);
-        auto old = gl_wrap::getViewport();
+        auto old = gl::getViewport();
         set_target();
         src.bind();
         float dx = 0;
@@ -274,28 +282,26 @@ public:
         std::array vertexData{x0,  y0,  x1,  y0,  x1,  y1,  x0,  y1,
                               0.F, 1.F, 1.F, 1.F, 1.F, 0.F, 0.F, 0.F};
         std::copy(src._uvs.begin(), src._uvs.end(), vertexData.begin() + 8);
-        gl_wrap::ArrayBuffer<GL_STREAM_DRAW> vbo{vertexData};
+        gl::ArrayBuffer<GL_STREAM_DRAW> vbo{vertexData};
 
         // TODO: It might be better to let Texture have its own program
         //       then to depend on ProgramCache.
-        auto& program = ProgramCache::get_instance()
+        auto program = ProgramCache::get_instance()
                             .get_program<ProgramCache::Textured,
                                          ProgramCache::NoTransform>();
         vbo.bind();
-        program.use();
-        auto pos = program.getAttribute("in_pos");
-        auto uv = program.getAttribute("in_uv");
+        program->use();
+        auto pos = program->getAttribute("in_pos");
+        auto uv = program->getAttribute("in_uv");
         pos.enable();
         uv.enable();
-        gl_wrap::vertexAttrib(pos, 2, gl_wrap::Type::Float, 0 * sizeof(GLfloat),
-                              0);
-        gl_wrap::vertexAttrib(uv, 2, gl_wrap::Type::Float, 0 * sizeof(GLfloat),
-                              8 * 4);
-        gl_wrap::drawArrays(gl_wrap::Primitive::TriangleFan, 0, 4);
+        gl::vertexAttrib(pos, 2, gl::Type::Float, 0 * sizeof(GLfloat), 0);
+        gl::vertexAttrib(uv, 2, gl::Type::Float, 0 * sizeof(GLfloat), 8 * 4);
+        gl::drawArrays(gl::Primitive::TriangleFan, 0, 4);
         pos.disable();
         uv.disable();
         glBindFramebuffer(GL_FRAMEBUFFER, fb);
-        gl_wrap::setViewport(old);
+        gl::setViewport(old);
     }
 
     void set_target() const { tex->set_target(); }
@@ -377,4 +383,4 @@ public:
     }
 };
 
-} // namespace gl_wrap
+} // namespace gl
